/*
 *	ccryptgui, QT based graphical user interface to ccrypt
 *
 *	Copyright (C) 2011  Martin Hilscher
 *
 *	This program is free software; you can redistribute it and/or modify it under the
 *	terms of the GNU General Public License as published by theFree Software Foundation;
 *	either version 3 of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *	PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License along with this
 *	program; if not, see <http://www.gnu.org/licenses/>.
 *
 *	------------------------------------------------------------------
 *
 *	ccryptgui.h -  base definitions and defining base-class 
 */


#ifndef CCRYPTGUI_H
#define CCRYPTGUI_H

#include "ui_mainwindow.h"
#include "ui_prefwindow.h"
#include "ui_pwdwindow.h"
#include "ui_aboutwindow.h"
#include <QtGui/QFileDialog>

// Defines



#ifdef WIN
   #include "windows.h"
#endif

#define COPYRIGHT			"Copyright(C) 2013  Martin Hilscher"
#define VERSION				"0.4.0"
#define VERSIONDATE			"29.04.2013"

#define LOADLANG_ERR_PROG_PATH          0x1     // Error: no Program Path found
#define LOADLANG_ERR_ELIM_BIN_PATH      0x2     // Error: could not eliminate bin-path
#define LOADLANG_ERR_TRANSL_FILE        0x4     // Error: could not found translation file
#define LOADLANG_DE                     0x8     // Load German Language
#define LOADLANG_ES                     0x10    // Load Spanish Language


#ifdef WIN
  #define TR_FILE_PATH			"share\\ccryptgui\\translations\\"
  #define TR_FILE_DE			"ccryptgui_de"
  #define TR_FILE_ES			"ccryptgui_es"
  #define URL_HELP_EN			"share\\ccryptgui\\help\\help_en.pdf"
  #define URL_HELP_DE			"share\\ccryptgui\\help\\help_de.pdf"
  #define URL_HELP_ES			"share\\ccryptgui\\help\\help_es.pdf"
#else //LINUX
   #define TR_FILE_PATH			"share/ccryptgui/translations/"
   #define TR_FILE_DE			"ccryptgui_de"
   #define TR_FILE_ES			"ccryptgui_es"
   #define URL_HELP_EN			"share/ccryptgui/help/help_en.pdf"
   #define URL_HELP_DE			"share/ccryptgui/help/help_de.pdf"
   #define URL_HELP_ES			"share/ccryptgui/help/help_es.pdf"
#endif

#define URL_HOMEPAGE			"http://www.softwaremustbefree.eu"

#ifdef WIN
   #define INI_FILE_NAME		"\\ccryptgui.ini"
   #define TMP_DIR_NAME			"ccryptgui_tmp"
   #define DIR_SLASH			"\\"
   #define DIR_SLASH_CHAR		'\\'
   #define DIR_SLASH_LINUX              "/"
#else //LINUX
   #define INI_FILE_NAME		"/.ccryptgui"
   #define TMP_DIR_NAME			".ccryptgui-tmp"
   #define DIR_SLASH			"/"
   #define DIR_SLASH_CHAR		'/'
#endif

#define MAX_PATH_LENGTH			256
#define MAX_CMD_LENGTH			256
#define MAX_PWD_LENGTH			128
#define N_CMD_ARGS			20

#define NO_DEBUG_MODE			0
#define DEBUG_MODE			1

#define CRYPT_MODE_DECRYPT		1
#define CRYPT_MODE_ENCRYPT		2
#define CRYPT_MODE_OPEN			3

#define PWD_OK				2
#define ERR_PWD				1

#define FILE_DIALOG_FILES_MODE		0
#define FILE_DIALOG_DIR_MODE		1

#ifdef WIN
   #define DEFAULT_FONT_SIZE		10
   #define DEFAULT_FONT_SIZE_HEADER	12
#else //LINUX
   #define DEFAULT_FONT_SIZE		10
   #define DEFAULT_FONT_SIZE_HEADER	16
#endif

#ifdef WIN
   #define DEFAULT_LOG_FILE_NAME	"ccryptgui.log"
   #define DEFAULT_CMD_ENCRYPT		"ccrypt -e"
   #define DEFAULT_CMD_DECRYPT		"ccrypt -d"
#else //LINUX
   #define DEFAULT_LOG_FILE_NAME	".ccryptgui.log"
   #define DEFAULT_CMD_ENCRYPT		"ccrypt -e"
   #define DEFAULT_CMD_DECRYPT		"ccrypt -d"
#endif

#define DEFAULT_ENABLE_LOG_FILE		1


#ifdef WIN
    #define SUBKEY_EIGENE_DATEIN "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\User Shell Folders"
    #define KEY_VALUE_EIGENE_DATEIN "Personal"
#endif



// Typedefs

#ifdef WIN
    // Definition der DLL-Funktion, die verwendet werden soll
    typedef LONG (*GetRegVal)( HKEY, char*, char*, char*, unsigned long);
#endif


// Base-Class

class CCryptGui : public QWidget
{
   Q_OBJECT

public:

   QMainWindow    *mainwindow;
   Ui_MainWindow  *ui_main;

   QDialog        *prefwindow;
   Ui_PrefWindow  *ui_pref;

   QDialog        *pwdwindow;
   Ui_PwdWindow   *ui_pwd;

   QDialog        *aboutwindow;
   Ui_AboutWindow *ui_about;

   QFileDialog    *filedialog;


   int  debug_mode;
   int  mrkLoadLang;

   QByteArray thisAppPath;
   QByteArray binPath;

private:

   int  crypt_mode;
   int  err_ccrypt;
   int  enable_log_file;
   int  enable_log;
   FILE* logfile;

   QByteArray curr_path;
   QByteArray init_path;
   QByteArray temp_path;
   QByteArray log_file_name;

   QByteArray cmd_encrypt;
   QByteArray cmd_decrypt;

   QByteArray cmd_encrypt_args;
   QByteArray cmd_decrypt_args;

   char* ptr_cmd_encrypt_args[N_CMD_ARGS];
   char* ptr_cmd_decrypt_args[N_CMD_ARGS];

   int  n_cmd_encrypt_args;
   int  n_cmd_decrypt_args;

   int  font_size;
   int  font_size_header;

   int  n_files;
   int  filedialog_mode;
   QStringList filelist;


public:

// Initialize
   CCryptGui( );

// First messages
   void messages_from_main(); 

// Set command arguments
   void set_cmd_args( );


public slots:
     
   void encrypt( );
   void decrypt( );
   void open( );
   void open_mode( );
   void change_pref( );
   void set_default( );
   void start_crypt( );
   void password_Ok( );
   void password_Abort( );
   void disable_buttons( );
   void enable_buttons( );
   void set_fonts( );
   void remove_temp_dir( );
   void call_homepage( );
   void call_help( );


private:

// Write preference data to file
   void save_ini ( );

// Read preference data from file
   void get_ini ( );

// Set widgets of preference-window
   void set_pref_widgets( );
 
// Write debug infos to console
   void debug_infos( );

// Copy choosen files to tmp-directory
   int cpy_files_to_tmp ( int );

// Execute the application to show the file(s)
   int execute_appl ( );

// Init log-file
   void init_logfile ( );

// Close log-file
   void close_logfile ( );

// Write Messages to Mainwindow and to log-file (if selected)
   void write_messages( const char* );

// Execute crypt
   void crypt_exe ( );

// Call a url-page via standard application ( e.g. browser)
   int call_app( const char*, char* const*, const char* );

};
   

#endif	//CCRYPTGUI_H
