/*
 *	ccryptgui, QT based graphical user interface to ccrypt
 *
 *	Copyright (C) 2011  Martin Hilscher
 *
 *	This program is free software; you can redistribute it and/or modify it under the
 *	terms of the GNU General Public License as published by theFree Software Foundation;
 *	either version 3 of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *	PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License along with this
 *	program; if not, see <http://www.gnu.org/licenses/>.
 *
 *	-------------------------------------------------------------------
 *
 *	common.cpp -  common used functions	
 */
 

#include "ccryptgui.h"
#include "time.h"
#include "stdio.h"

#ifdef WIN
   #include "direct.h"
   #include "process.h"
   #include "shellapi.h"
#else //LINUX
   #include "unistd.h"
   #include "sys/wait.h"
   #include "sys/stat.h"
#endif


  // Write preference data to file
  void CCryptGui::save_ini ( )
  {

     FILE* inifile;
     char str[MAX_PATH_LENGTH];

     strncpy ( str, init_path, MAX_PATH_LENGTH );
     strncat ( str, INI_FILE_NAME, MAX_PATH_LENGTH );
     inifile = fopen( str, "w" );

     fprintf ( inifile, "#\n" );
     fprintf ( inifile, "# Config File ccryptgui - don't edit !\n" );
     fprintf ( inifile, "#\n\n" );
     fprintf ( inifile, "Version=" );
     fprintf ( inifile, VERSION );
     fprintf ( inifile, "\n" );
     fprintf ( inifile, "LogFileName=" );
     fprintf ( inifile, "%s\n", log_file_name.data() );
     fprintf ( inifile, "CmdEncrypt=" );
     fprintf ( inifile, "%s\n", cmd_encrypt.data() );
     fprintf ( inifile, "CmdDecrypt=" );
     fprintf ( inifile, "%s\n", cmd_decrypt.data() );
     fprintf ( inifile, "CurrentPath=" );
     fprintf ( inifile, "%s\n", curr_path.data() );
     fprintf ( inifile, "MrkLogFile=" );
     fprintf ( inifile, "%i\n", enable_log_file );
     fprintf ( inifile, "FontSize=" );
     fprintf ( inifile, "%i\n", font_size );
     fprintf ( inifile, "FontSizeHeader=" );
     fprintf ( inifile, "%i\n", font_size_header );

     fclose ( inifile ); 

  }


  // Read Preference data from ini-file
  void CCryptGui::get_ini ( )
  {

     FILE* inifile;
     char str[MAX_PATH_LENGTH];
     char *ptr;
     bool mkr_version_ok = TRUE;

     strncpy ( str, init_path.data(), MAX_PATH_LENGTH );
     strncat ( str, INI_FILE_NAME, MAX_PATH_LENGTH );

     if ( (inifile = fopen( str, "r" )) )
     {

         while ( !feof( inifile ) )
        {
           fgets ( str, MAX_PATH_LENGTH, inifile );
           if ( ( ptr = strchr ( str, '\n' ) ) )
              *ptr = 0;

           if ( *str )
           {
              if ( strstr( str, "Version=" ) )
              {
                 if ( strcmp ( str+8, VERSION ) )
                 {
                    ui_main->textEdit->setText( " **WARNING: Ini-File: false version" );
                    mkr_version_ok = FALSE;
                 }
              }

              if ( strstr( str, "LogFileName=" ) ) log_file_name = str+12;
 
              if ( strstr( str, "CmdEncrypt=" ) )  cmd_encrypt = str+11;

              if ( strstr( str, "CmdDecrypt=" ) )  cmd_decrypt = str+11;

              if ( strstr( str, "CurrentPath=" ) ) curr_path = str+12;
 
              if ( strstr( str, "MrkLogFile=" ) )  sscanf ( str+11, "%i", &enable_log_file );

              if ( strstr( str, "FontSize=" ) )  sscanf ( str+9, "%i", &font_size );

              if ( strstr( str, "FontSizeHeader=" ) )  sscanf ( str+15, "%i", &font_size_header );
           }

        }

        fclose ( inifile );

        if ( !mkr_version_ok ) save_ini();

     }
     else {

        set_default();
        save_ini();
     }
  }


  // First messages
  void CCryptGui::messages_from_main()  
  {
     // Messages from loading translation files
     if ( mrkLoadLang & LOADLANG_ERR_PROG_PATH )
        write_messages( "** ERROR: main: could not find path of this application" );

     if ( mrkLoadLang & LOADLANG_ERR_ELIM_BIN_PATH )
        write_messages( "** ERROR: main: could not eliminate bin-path" );

     if ( mrkLoadLang & LOADLANG_ERR_TRANSL_FILE )
        write_messages( "** ERROR: main: could not load translation file" );
  }



  // Set widgets of preference-window
  void CCryptGui::set_pref_widgets()  
  {
     ui_pref->lineEditE->setText( cmd_encrypt );
     ui_pref->lineEditD->setText( cmd_decrypt );
     ui_pref->lineEdit_log->setText( log_file_name );
     if (enable_log_file)
        ui_pref->radioButton_log->setChecked( TRUE );
     else
        ui_pref->radioButton_log->setChecked( FALSE );
     ui_pref->spinBox->setValue( font_size );
  }


  // Disable buttons of mainwindow
  void CCryptGui::disable_buttons( )
  {
     ui_main->pushButtonE->setEnabled( FALSE );
     ui_main->pushButtonD->setEnabled( FALSE );
     ui_main->pushButtonO->setEnabled( FALSE );
     ui_main->checkBox->setEnabled( FALSE );
  }
 
  // Enable buttons of mainwindow
  void CCryptGui::enable_buttons( )
  {
     ui_main->pushButtonE->setEnabled( TRUE );
     ui_main->pushButtonD->setEnabled( TRUE );
     ui_main->pushButtonO->setEnabled( TRUE );
     ui_main->checkBox->setEnabled( TRUE );
  }
 

  // Write debug infos to console
  void CCryptGui::debug_infos( void )
  {
     printf ("===================\n");
     printf ("filedialog_mode : %i\n", filedialog_mode);
     printf ("log_file_name   : %s\n", log_file_name.data() );
     printf ("cmd_encrypt     : %s\n", cmd_encrypt.data() );
     printf ("cmd_decrypt     : %s\n", cmd_decrypt.data() );
     printf ("curr_path       : %s\n", curr_path.data() );
     printf ("enable_log_file : %i\n", enable_log_file );
     printf ("n_files         : %i\n", n_files);
     printf ("filedialog_mode : %i\n", filedialog_mode);
  }


  // Set Command Arguments
  void CCryptGui::set_cmd_args( void )
  {
     char* ptr;
     int   i;

     cmd_encrypt_args  = cmd_encrypt;

     ptr = strtok( cmd_encrypt_args.data(), " " );

     ptr_cmd_encrypt_args[0] = ptr;
     for ( i = 1; i < (N_CMD_ARGS-5) ;i++ )
     {
        ptr = strtok( NULL, " " );
        ptr_cmd_encrypt_args[i] = ptr;
        if ( ptr == NULL )  break;
     }

     n_cmd_encrypt_args = i;

     cmd_decrypt_args  = cmd_decrypt;

     ptr = strtok( cmd_decrypt_args.data(), " " );
     ptr_cmd_decrypt_args[0] = ptr;
     for ( i = 1; i < (N_CMD_ARGS-5); i++ )
     {
        ptr = strtok( NULL, " " );
        ptr_cmd_decrypt_args[i] = ptr;
        if ( ptr == NULL )  break;
     }

     n_cmd_decrypt_args = i;
  }


  // Copy selected files to tmp-directory
  int CCryptGui::cpy_files_to_tmp ( int cnt )
  {
     QByteArray filename = (filelist.value(cnt)).toAscii();
     int pos;

#ifdef WIN
     // in "filename" set character "/" to "\" to satisfy "copy" instruction
     int last_pos = -1;
     pos  =  0;
     while ( ( pos = filename.indexOf( DIR_SLASH_LINUX, pos ) ) >= 0 )
     {
         filename[pos] = DIR_SLASH_CHAR;
         last_pos = pos;
     }

     QByteArray temp = "copy /Y ";
     temp += "\"";
     temp += filename;
     temp += "\" \"";
     temp += temp_path;
     temp += "\"";

     if ( system ( temp.data() ) )
     {
        return -1;
     }

     // "filename" will be removed until the last occurrence of a slash
     // if no slash, last_pos = -1.

     temp  = temp_path;
     temp += DIR_SLASH;
     if ( last_pos < 0 )
        temp += filename;
     else
        temp += filename.right( filename.length() - last_pos - 1 );


#else
     // "filename" will be removed until the last occurrence of a slash
     // if no slash, funktion "QByteArray::lastIndex" returns -1
     pos   = filename.lastIndexOf( DIR_SLASH );

     QByteArray temp  = temp_path;
     temp += DIR_SLASH;
     if ( pos < 0 )
        temp += filename;
     else
        temp += filename.right( filename.length() - pos );

     char* args[] = { (QByteArray( "cp" )).data(),
                      (QByteArray( "-f" )).data(), filename.data(), temp.data(), NULL };

     if ( call_app ( "cp", args, "copy files to temp" ) )
     {
        return -1;
     } 

     // change access mode of files
     chmod( temp, S_IRUSR | S_IWUSR );
#endif

     // filelist must changed to temp-path
     filelist.replace( cnt, temp );

     return 0;
  } 


  // Execute the application to open the file(s)
  int CCryptGui::execute_appl ( )
  {
     int cnt;
     int pos;
     QByteArray file;
#ifndef WIN  // Linux
     char* args[] = { (QByteArray( "xdg-open" )).data(), NULL, NULL };
#endif

     int cnt_success = 0;

     for ( cnt = 0; cnt < n_files; cnt++ )
     {
        file = (filelist.value(cnt)).toAscii();

        // "ccrypt" normally removes the extension ".cpt" of the filename in case of decrypting
        pos  = file.lastIndexOf( ".cpt" );
        if ( pos >= 0 )
            file.truncate( pos );


#ifdef WIN
        if ( (int) ShellExecuteA( NULL, "open", file.data(), NULL, NULL, SW_SHOWNORMAL ) > 32 )
        {
            cnt_success++;
        }
#else  // Linux
        args[1] = file.data();

        if (!( call_app ( "xdg-open", args, "application" ) ) )
        {
           cnt_success++;
        } 
#endif
    }

     return cnt_success;
  }


  // Init log-file
  void CCryptGui::init_logfile ( )
  {

     QByteArray temp;

     time_t *now = new time_t;

     temp  = init_path;
     temp += DIR_SLASH;
     temp += log_file_name;

     logfile = fopen ( temp.data(), "w" );
     if ( logfile == NULL )
     {
        ui_main->textEdit->append( QString::fromUtf8("** Error while opening log-file") );
        enable_log = 0;
     }
     else
     {
        ui_main->textEdit->append( QString::fromUtf8("** log-file opened") );
        enable_log = 1;
        fprintf ( logfile, "#\n# ***** ccryptgui *****" );
        fprintf ( logfile, "\n#\n# ");
        fprintf ( logfile, COPYRIGHT );
        fprintf ( logfile, "\n#\n# Log file: %s\n#\n\n", temp.data() );

        time ( now );
        fprintf ( logfile, "** Start time: %s\n\n", ctime ( now ) );
     }

  }


  // Close log-file
  void CCryptGui::close_logfile ( )
  {
     time_t *now = new time_t;

     time ( now );
     fprintf ( logfile, "\n** End time: %s\n", ctime ( now ) );

     if ( fclose ( logfile ) )
        ui_main->textEdit->append( QString::fromAscii( "** error while closing log-file" ) );
     else
        ui_main->textEdit->append( QString::fromAscii( "** log-file closed" ) );

     enable_log = 0;
  }


  // Write Messages to Mainwindow and to log-file (if selected)
  void CCryptGui::write_messages( const char* pmsg )
  {
      ui_main->textEdit->append( QString::fromAscii( QByteArray( pmsg ) ) );

      if ( enable_log ) 
         fprintf( logfile, "%s\n", pmsg );

  }


  // Set fonts
  void CCryptGui::set_fonts (  )
  {
     font_size = ui_pref->spinBox->value();

     QFont font;
     font.setPointSize(font_size);

     mainwindow->setFont(font);
     prefwindow->setFont(font);
     pwdwindow->setFont(font);
     aboutwindow->setFont(font);
     filedialog->setFont(font);

     QFont font_bold;
     font_bold.setPointSize(font_size);
     font_bold.setBold(true);
     ui_pwd->label1->setFont(font_bold);

     font_size_header = ui_pref->spinBox_2->value();

     QFont font_header;
     font_header.setPointSize(font_size_header);
     font_header.setItalic(true);
     ui_main->label->setFont(font_header);
     ui_about->label->setFont(font_header);

     save_ini();

  }


  // Call application
  int CCryptGui::call_app( const char* cmd, char* const * args, const char* msg )
  {
     int status;

     QByteArray temp;


#ifdef WIN

     // Change path to bin-Path
     // (spawnv(p) cannot handle blanks in the path parameter)
     temp  = thisAppPath;
     temp += binPath;
     _chdir( temp.data() );

     status = spawnvp( _P_WAIT, cmd, args );

     if ( status )
     {
        temp  = "** ERROR: call ";
        temp += msg;
        temp += ": exec-error: ERR = ";
        temp += QByteArray::number( status );
        write_messages( temp.data() );
     }

#else //LINUX 

     pid_t pid;

     switch ( pid = fork() )
     {
        // fork-error
        case -1:

           temp  = "** ERROR: call ";
           temp += msg;
           temp += ": fork-error";
           write_messages( temp.data() );
           return -1;

        // child: open application
        case 0:

           execvp( cmd, args );
           exit(127);

        // parents
        default:
           wait( &status );

           if ( status )
           {
              temp  = "** ERROR: call ";
              temp += msg;
              temp += ": exec-error: ERR = ";
              temp += QByteArray::number( status );
              write_messages( temp.data() );
           }

     }

#endif

     return status;

  }


  // Call homepage-page
  void CCryptGui::call_homepage( )
  {
#ifdef WIN
     ShellExecuteA( NULL, "open", URL_HOMEPAGE, NULL, NULL, SW_SHOWNORMAL );
#else  //Linux
     char* args[] = { (QByteArray( "xdg-open" )).data(), (QByteArray( URL_HOMEPAGE )).data(), NULL };
     call_app( "xdg-open", args, "call_homepage" );
#endif
  }


  // Call help-page
  void CCryptGui::call_help( )
  {
     QByteArray helpPath = thisAppPath + URL_HELP_EN;

     if ( mrkLoadLang == LOADLANG_DE)
        helpPath = thisAppPath + URL_HELP_DE;

     if ( mrkLoadLang == LOADLANG_ES)
        helpPath = thisAppPath + URL_HELP_ES;

#ifdef WIN
     ShellExecuteA( NULL, "open", helpPath.data(), NULL, NULL, SW_SHOWNORMAL );
#else  //Linux
     char* args[] = { (QByteArray( "xdg-open" )).data(), helpPath.data(), NULL };
     call_app( "xdg-open", args, "call_help" );
#endif
  }





  
